package com.google.code.ginmvp.autoplacehistorymapper.server;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import com.google.code.ginmvp.autoplacehistorymapper.client.UseTokenizer;
import com.google.gwt.core.ext.Generator;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.NotFoundException;
import com.google.gwt.place.shared.PlaceTokenizer;
import com.google.gwt.place.shared.Prefix;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

/**
 * 
 * The generator that supports the AutoPlaceHistoryMapper. 
 * 
 * <br /><br />
 * 
 * This generator is used internally by the AutoPlaceHistoryMapper to dynamically generate
 * a factory for all of the Place types that are detected within your application.
 * 
 * <br /><br />
 * 
 * The generator will first look to see if your place has a public static inner {@link PlaceTokenizer} class
 * named &quot;Tokenizer&quot.  If so, it will be passed in a @WithTokenizers list to the default PlaceHistoryMapper
 * generator.  If there is no PlaceTokenizer class found for a place, the generator then handles the Place by creating
 * a factory to instantiate the Place type - using the same prefix logic that is used by the default PlaceHistoryMapper.
 * 
 * <br /><br />
 * 
 * To customize the Prefix used for a place, specify the @{@link Prefix} annotation on either the Place class or it's
 * inner {@link PlaceTokenizer}.
 * 
 * @author slynn1324
 *
 */
public class AutoPlaceHistoryMapperGeneratedDelegateGenerator extends Generator {

	@Override
	public String generate(TreeLogger logger, GeneratorContext context,
			String typeName) throws UnableToCompleteException {
		
		try{
			
			JClassType classType;
			classType = context.getTypeOracle().getType(typeName);
			
			SourceWriter src = getSourceWriter(classType, context, logger);
			if ( src != null ){

			List<String> tokenizers = new ArrayList<String>();
			
			List<JClassType> typesToCreate = new ArrayList<JClassType>();
			
			JClassType[] types = context.getTypeOracle().getTypes();
			JClassType placeType = context.getTypeOracle().getType("com.google.gwt.place.shared.Place");
	        for (JClassType type : types) {
	            if ( type.isAssignableTo(placeType) && !type.isAbstract() && !type.getQualifiedSourceName().equals("com.google.gwt.place.shared.Place")){
	            	
	            	
	            	if (type.isAnnotationPresent(UseTokenizer.class)){
	            		
	            		Class<? extends PlaceTokenizer<?>> tokenizerClass = type.getAnnotation(UseTokenizer.class).value();
	            		JClassType tokenizerType = context.getTypeOracle().getType(tokenizerClass.getCanonicalName());
	            		tokenizers.add(tokenizerType.getQualifiedSourceName() + ".class");
	            		
	            	} else {
	            		
		            	try{
	//	            		System.out.println("looking for: " + type.getQualifiedSourceName() + ".Tokenizer");
		            		JClassType tokenizerType = context.getTypeOracle().getType(type.getQualifiedSourceName() + ".Tokenizer");
		            		tokenizers.add(tokenizerType.getQualifiedSourceName() + ".class");
		            	} catch (NotFoundException e){
		            		typesToCreate.add(type);
		            	}
	            	}
	            	
	            	
	            }
	        }
	        
//	        for ( JClassType type : typesToCreate ){
//	        	System.out.println("creating factory to handle places: " + type.getQualifiedSourceName());
//	        }
	        
	        
	        //
	        // Creating a standard PlaceHistoryMapper interface using the @WithTokenizers annotation,
	        // which will be re-generated by the GWT compiler to take advantage of the original PlaceHistoryMapper
	        // generator.
	        //
	        
	        src.println("@WithTokenizers({");
	        int count = 0;
	        for ( String tokenizer : tokenizers ){
//	        	System.out.println("using the following tokenizers: " + tokenizer);
            	if ( count > 0 ){
            		src.print(",");
            	}
            	src.println(tokenizer);
            	++count;
	        }
	        src.println("})");
	        
			src.println("public static interface TokenizedAutoPlaceHistoryMapperDelegate extends PlaceHistoryMapper {}");

			// instance of the standard PlaceHistoryMapper - the GWT.create() call will cause 
			// the GWT compiler to re-generate a PlaceHistoryMapper used here.
			src.println("TokenizedAutoPlaceHistoryMapperDelegate tokenizedDelegate = GWT.create(TokenizedAutoPlaceHistoryMapperDelegate.class);");
			
			
			// implement the getPlace() method
			src.println("public Place getPlace(String token) {");
			src.println("return getDelegate().beforeReturnPlace(getPlaceHelper(token), token);");
			src.println("}");
			
			
			
			src.println("private Place getPlaceHelper(String token) {  ");
			src.println("if (token != null ){");
			for(JClassType typeToCreate : typesToCreate ){
				String prefix = typeToCreate.getName();
				if ( typeToCreate.isAnnotationPresent(Prefix.class)){
					prefix = typeToCreate.getAnnotation(Prefix.class).value();
				}
				
				String fullName = typeToCreate.getQualifiedSourceName();
				src.println("if ( token.startsWith(\"" + prefix + ":\") ) { return new " + fullName + "(); }");
			}
			src.println("} return tokenizedDelegate.getPlace(token);  }");

			src.println("public String getToken(Place place) { ");
			
			src.println("if (place != null ){");
			for(JClassType typeToCreate : typesToCreate ){
				String fullName = typeToCreate.getQualifiedSourceName();
				String prefix = typeToCreate.getName();
				if ( typeToCreate.isAnnotationPresent(Prefix.class)){
					prefix = typeToCreate.getAnnotation(Prefix.class).value();
				}
				prefix = prefix + ":";
				src.println("if ( place.getClass() == " + fullName + ".class ){ return \"" + prefix + "\" + getDelegate().getDefaultToken(place); }");
			}
			src.println("} return tokenizedDelegate.getToken(place); }");
			
			src.commit(logger);
			
			return typeName + "_Generated";
			}
		} catch (Exception e){
			e.printStackTrace();
		}
		return typeName + "_Generated";
		
	}
	
	
	protected SourceWriter getSourceWriter(JClassType classType,
			GeneratorContext context, TreeLogger logger) {
		String packageName = classType.getPackage().getName();
		String simpleName = classType.getSimpleSourceName() + "_Generated";
		ClassSourceFileComposerFactory composer = new ClassSourceFileComposerFactory(
				packageName, simpleName);
		composer.setSuperclass(classType.getName());
		composer.addImplementedInterface("com.google.gwt.place.shared.PlaceHistoryMapper");

		// imports used in the generated code
		composer.addImport("com.google.gwt.core.client.GWT");
		composer.addImport("com.google.gwt.place.shared.Place");
		composer.addImport("com.google.gwt.place.shared.PlaceHistoryMapper");
		composer.addImport("com.google.gwt.place.shared.WithTokenizers");
		composer.addImport("com.google.gwt.place.shared.PlaceTokenizer");

		PrintWriter printWriter = context.tryCreate(logger, packageName,
				simpleName);
		if (printWriter == null) {
			return null;
		} else {
			SourceWriter sw = composer.createSourceWriter(context, printWriter);
			return sw;
		}
	}

}
